@@ -12,16 +12,17 @@ Authors: Henning O. Sorensen & Erik Knudsen
          and Jon Wright, Jerome Kieffer: ESRF
 
 """
-from __future__ import with_statement
+# get ready for python3
+from __future__ import with_statement, print_function
 import os, types, logging, sys, tempfile
 logger = logging.getLogger("fabioimage")
 import numpy
 try:
-    import Image
+    from PIL import Image
 except ImportError:
     logger.warning("PIL is not installed ... trying to do without")
     Image = None
-import fabioutils, converters
+from . import fabioutils, converters
 
 
 class fabioimage(object):
@@ -374,6 +375,7 @@ class fabioimage(object):
         fileObject = None
         self.filename = fname
         self.filenumber = fabioutils.extract_filenumber(fname)
+
         if hasattr(fname, "read") and hasattr(fname, "write"):
             # It is already something we can use
             return fname
@@ -407,7 +409,7 @@ class fabioimage(object):
                            python_uncompress,
                            mode='rb'):
         """
-        Try to transparently handle gzip / bzip without always getting python
+        Try to transparently handle gzip / bzip2 without always getting python
         performance
         """
         # assert that python modules are always OK based on performance benchmark
@@ -550,7 +552,7 @@ def test():
     ftest.close()
     clean()
 
-    print "Passed in", time.time() - start, "s"
+    print ("Passed in %s s" % (time.time() - start))
 
 if __name__ == '__main__':
     test()
