@@ -1,14 +1,23 @@
 #!/usr/bin/env python
-#coding: utf8
+#coding: utf-8
 
 """
 General purpose utilities functions for fabio
 """
-from __future__ import with_statement
+# get ready for python3
+from __future__ import with_statement, print_function
 import re, os, logging, threading, sys
-import StringIO as stringIO
+if sys.version_info < (3,):
+    import StringIO as io
+    FileIO = file
+else:
+    import io
+    from io import FileIO
 logger = logging.getLogger("fabioutils")
-from compression import bz2, gzip
+from .compression import bz2, gzip
+import traceback
+from math import ceil
+
 
 
 FILETYPES = {
@@ -21,7 +30,7 @@ FILETYPES = {
     'pbm'    : ['pnm'],
     'tif'    : ['tif'],
     'tiff'   : ['tif'],
-    'img'    : ['adsc', 'OXD', 'HiPiC'],
+    'img'    : ['adsc', 'OXD', 'HiPiC', 'raxis'],
     'mccd'   : ['marccd'],
     'mar2300': ['mar345'],
     'sfrm'   : ['bruker100'],
@@ -35,7 +44,7 @@ FILETYPES = {
              }
 
 # Add bzipped and gzipped
-for key in FILETYPES.keys():
+for key in list(FILETYPES.keys()):
     FILETYPES[key + ".bz2"] = FILETYPES[key]
     FILETYPES[key + ".gz"] = FILETYPES[key]
 
@@ -55,19 +64,42 @@ try:
         COMPRESSORS['.gz'] = 'gzip -dc '
     else:
         COMPRESSORS['.gz'] = None
-except:
+except Exception:
     COMPRESSORS['.gz'] = None
 
 try:
     lines = os.popen("bzip2 -h 2>&1").read()
-    # Looking for "usage" 
+    # Looking for "usage"
     if "sage" in lines:
         COMPRESSORS['.bz2'] = 'bzip2 -dc '
     else:
         COMPRESSORS['.bz2'] = None
-except:
+except Exception:
     COMPRESSORS['.bz2'] = None
 
+def deprecated(func):
+    """
+    used to deprecate a function/method: prints a lot of warning messages to enforce the modifaction of the code
+    """
+    def wrapper(*arg, **kw):
+        """
+        decorator that deprecates the use of a function
+        """
+        logger.warning("%s is Deprecated !!! %s" % (func.func_name, os.linesep.join([""] + traceback.format_stack()[:-1])))
+        return func(*arg, **kw)
+    return wrapper
+
+def pad(mystr, pattern=" ", size=80):
+    """
+    Performs the padding of the string to the right size with the right pattern
+    """
+    size = int(size)
+    padded_size = int(ceil(float(len(mystr)) / size) * size)
+    if len(pattern) == 1:
+        return mystr.ljust(padded_size, pattern)
+    else:
+        return (mystr + pattern * int(ceil(float(padded_size - len(mystr)) / len(pattern))))[:padded_size]
+
 
 def getnum(name):
     """
@@ -80,17 +112,36 @@ def getnum(name):
     except ValueError:
         return None
 
-class filename_object:
+class FilenameObject(object):
     """
-    The 'meaning' of a filename
+    The 'meaning' of a filename ...
     """
-    def __init__(self, stem,
+    def __init__(self, stem=None,
             num=None,
             directory=None,
             format=None,
             extension=None,
             postnum=None,
-            digits=4):
+            digits=4,
+            filename=None):
+        """
+        This class can either be instanciated by a set of parameters like  directory, prefix, num, extension, ...
+
+        @param stem: the stem is a kind of prefix (str)
+        @param num: image number in the serie (int)
+        @param directory: name of the directory (str)
+        @param format: ??
+        @param extension:
+        @param postnum:
+        @param digits: Number of digits used to print num
+
+        Alternative constructor:
+
+        @param filename: fullpath of an image file to be deconstructed into directory, prefix, num, extension, ...
+
+        """
+
+
         self.stem = stem
         self.num = num
         self.format = format
@@ -98,6 +149,10 @@ class filename_object:
         self.digits = digits
         self.postnum = postnum
         self.directory = directory
+        self.compressed = None
+        if filename is not None:
+            self.deconstruct_filename(filename)
+
 
     def str(self):
         """ Return a string representation """
@@ -111,7 +166,7 @@ class filename_object:
                     self.postnum ,
                     self.digits ,
                     self.directory ] ])
-
+    __repr__ = str
 
     def tostring(self):
         """
@@ -130,6 +185,76 @@ class filename_object:
         return name
 
 
+    def deconstruct_filename(self, filename):
+        """
+        Break up a filename to get image type and number
+        """
+        direc, name = os.path.split(filename)
+        direc = direc or None
+        parts = name.split(".")
+        compressed = False
+        stem = parts[0]
+        extn = ""
+        postnum = ""
+        ndigit = 4
+        num = None
+        typ = None
+        if parts[-1] in ["gz", "bz2"]:
+            extn = "." + parts[-1]
+            parts = parts[:-1]
+            compressed = True
+        if parts[-1] in FILETYPES.keys():
+            typ = FILETYPES[parts[-1]]
+            extn = "." + parts[-1] + extn
+            try:
+                stem, numstring, postnum = numstem(".".join(parts[:-1]))
+                num = int(numstring)
+                ndigit = len(numstring)
+            except Exception as err:
+                # There is no number - hence make num be None, not 0
+                logger.debug("l176: %s" % err)
+                num = None
+                stem = "".join(parts[:-1])
+        else:
+            # Probably two type left
+            if len(parts) == 1:
+                # Probably GE format stem_numb
+                parts2 = parts[0].split("_")
+                if parts2[-1].isdigit():
+                    num = int(parts2[-1])
+                    ndigit = len(parts2[-1])
+                    typ = ['GE']
+                    stem = "_".join(parts2[:-1]) + "_"
+            else:
+                try:
+                    num = int(parts[-1])
+                    ndigit = len(parts[-1])
+                    typ = ['bruker']
+                    stem = ".".join(parts[:-1]) + "."
+                except Exception as err:
+                    logger.debug("l196: %s" % err)
+                    typ = None
+                    extn = "." + parts[-1] + extn
+                    numstring = ""
+                    try:
+                        stem , numstring, postnum = numstem(".".join(parts[:-1]))
+                    except Exception as err:
+                        logger.debug("l202: %s" % err)
+                        raise
+                    if numstring.isdigit():
+                        num = int(numstring)
+                        ndigit = len(numstring)
+                #            raise Exception("Cannot decode "+filename)
+
+        self.stem = stem
+        self.num = num
+        self.directory = direc
+        self.format = typ
+        self.extension = extn
+        self.postnum = postnum
+        self.digits = ndigit
+        self.compressed = compressed
+
 def numstem(name):
     """ cant see how to do without reversing strings
     Match 1 or more digits going backwards from the end of the string
@@ -140,114 +265,56 @@ def numstem(name):
         res = reg.match(name).groups()
         #res = reg.match(name[::-1]).groups()
         #return [ r[::-1] for r in res[::-1]]
-        if len(res[0]) == len(res[1]) == 0: # Hack for file without number 
+        if len(res[0]) == len(res[1]) == 0: # Hack for file without number
             return [res[2], '', '']
         return [ r for r in res]
     except AttributeError: # no digits found
         return [name, "", ""]
 
+#@deprecated
 def deconstruct_filename(filename):
     """
-    Break up a filename to get image type and number
+    Function for backward compatibility.
+    Deprecated
     """
-    direc, name = os.path.split(filename)
-    direc = direc or None
-    parts = name.split(".")
-    compressed = False
-    stem = parts[0]
-    extn = ""
-    postnum = ""
-    ndigit = 4
-    num = None
-    typ = None
-    if parts[-1] in ["gz", "bz2"]:
-        extn = "." + parts[-1]
-        parts = parts[:-1]
-        compressed = True
-    if parts[-1] in FILETYPES.keys():
-        typ = FILETYPES[parts[-1]]
-        extn = "." + parts[-1] + extn
-        try:
-            stem, numstring, postnum = numstem(".".join(parts[:-1]))
-            num = int(numstring)
-            ndigit = len(numstring)
-        except Exception, err:
-            # There is no number - hence make num be None, not 0
-            logger.debug("l176: %s" % err)
-            num = None
-            stem = "".join(parts[:-1])
-    else:
-        # Probably two type left
-        if len(parts) == 1:
-            # Probably GE format stem_numb
-            parts2 = parts[0].split("_")
-            if parts2[-1].isdigit():
-                num = int(parts2[-1])
-                ndigit = len(parts2[-1])
-                typ = ['GE']
-                stem = "_".join(parts2[:-1]) + "_"
-        else:
-            try:
-                num = int(parts[-1])
-                ndigit = len(parts[-1])
-                typ = ['bruker']
-                stem = ".".join(parts[:-1]) + "."
-            except Exception, err:
-                logger.debug("l196: %s" % err)
-                typ = None
-                extn = "." + parts[-1] + extn
-                try:
-                    stem , numstring, postnum = numstem(".".join(parts[:-1]))
-                    num = int(numstring)
-                    ndigit = len(numstring)
-                except Exception, err:
-                    logger.debug("l204: %s" % err)
-                    raise
-            #            raise Exception("Cannot decode "+filename)
-    obj = filename_object(stem,
-            num=num,
-            directory=direc,
-            format=typ,
-            extension=extn,
-            postnum=postnum,
-            digits=ndigit)
-    return obj
+    return FilenameObject(filename=filename)
 
-def construct_filename(filename, frame):
+def construct_filename(filename, frame=None):
     "Try to construct the filename for a given frame"
-    fo = deconstruct_filename(filename)
-    fo.num = frame
-    return fo.tostring()
+    fobj = FilenameObject(filename=filename)
+    if frame is not None:
+        fobj.num = frame
+    return fobj.tostring()
 
 def next_filename(name, padding=True):
     """ increment number """
-    obj = deconstruct_filename(name)
-    obj.num += 1
+    fobj = FilenameObject(filename=name)
+    fobj.num += 1
     if not padding:
-        obj.digits = 0
-    return obj.tostring()
+        fobj.digits = 0
+    return fobj.tostring()
 
 def previous_filename(name, padding=True):
     """ decrement number """
-    obj = deconstruct_filename(name)
-    obj.num -= 1
+    fobj = FilenameObject(filename=name)
+    fobj.num -= 1
     if not padding:
-        obj.digits = 0
-    return obj.tostring()
+        fobj.digits = 0
+    return fobj.tostring()
 
 def jump_filename(name, num, padding=True):
     """ jump to number """
-    obj = deconstruct_filename(name)
-    obj.num = num
+    fobj = FilenameObject(filename=name)
+    fobj.num = num
     if not padding:
-        obj.digits = 0
-    return obj.tostring()
+        fobj.digits = 0
+    return fobj.tostring()
 
 
 def extract_filenumber(name):
     """ extract file number """
-    obj = deconstruct_filename(name)
-    return obj.num
+    fobj = FilenameObject(filename=name)
+    return fobj.num
 
 def isAscii(name, listExcluded=None):
     """
@@ -287,9 +354,9 @@ def toAscii(name, excluded=None):
     return "".join(out)
 
 def nice_int(s):
-    """ 
-    Workaround that int('1.0') raises an exception 
-    
+    """
+    Workaround that int('1.0') raises an exception
+
     @param s: string to be converted to integer
     """
     try:
@@ -298,14 +365,14 @@ def nice_int(s):
         return int(float(s))
 
 
-class StringIO(stringIO.StringIO):
+class StringIO(io.StringIO):
     """
     just an interface providing the name and mode property to a StringIO
 
     BugFix for MacOSX mainly
     """
     def __init__(self, data, fname=None, mode="r"):
-        stringIO.StringIO.__init__(self, data)
+        io.StringIO.__init__(self, data)
         self.closed = False
         if fname == None:
             self.name = "fabioStream"
@@ -314,7 +381,7 @@ class StringIO(stringIO.StringIO):
         self.mode = mode
         self.lock = threading.Semaphore()
         self.__size = None
-    
+
     def getSize(self):
         if self.__size is None:
             logger.debug("Measuring size of %s" % self.name)
@@ -328,13 +395,13 @@ class StringIO(stringIO.StringIO):
         self.__size = size
     size = property(getSize, setSize)
 
-class File(file):
+class File(FileIO):
     """
     wrapper for "file" with locking
     """
     def __init__(self, name, mode="rb", buffering=0):
         """file(name[, mode[, buffering]]) -> file object
-            
+
         Open a file.  The mode can be 'r', 'w' or 'a' for reading (default),
         writing or appending.  The file will be created if it doesn't exist
         when opened for writing or appending; it will be truncated when
@@ -348,10 +415,10 @@ class File(file):
         in Python.  Also, a file so opened gains the attribute 'newlines';
         the value for this attribute is one of None (no newline read yet),
         '\r', '\n', '\r\n' or a tuple containing all the newline types seen.
-        
+
         'U' cannot be combined with 'w' or '+' mode.
         """
-        file.__init__(self, name, mode, buffering)
+        FileIO.__init__(self, name, mode, buffering)
         self.lock = threading.Semaphore()
         self.__size = None
     def getSize(self):
@@ -365,8 +432,16 @@ class File(file):
         return self.__size
     def setSize(self, size):
         self.__size = size
+    def __exit__(self, *args, **kwargs):
+        """
+        Close the file.
+        """
+        return FileIO.close(self)
+    def __enter__(self, *args, **kwargs):
+        return self
     size = property(getSize, setSize)
 
+
 class UnknownCompressedFile(File):
     """
     wrapper for "File" with locking
@@ -380,32 +455,32 @@ if gzip is None:
 else:
     class GzipFile(gzip.GzipFile):
         """
-        Just a wrapper forgzip.GzipFile providing the correct seek capabilities for python 2.5   
+        Just a wrapper forgzip.GzipFile providing the correct seek capabilities for python 2.5
         """
         def __init__(self, filename=None, mode=None, compresslevel=9, fileobj=None):
             """
             Wrapper with locking for constructor for the GzipFile class.
-            
+
             At least one of fileobj and filename must be given a
             non-trivial value.
-            
+
             The new class instance is based on fileobj, which can be a regular
             file, a StringIO object, or any other object which simulates a file.
             It defaults to None, in which case filename is opened to provide
             a file object.
-            
+
             When fileobj is not None, the filename argument is only used to be
             included in the gzip file header, which may includes the original
             filename of the uncompressed file.  It defaults to the filename of
             fileobj, if discernible; otherwise, it defaults to the empty string,
             and in this case the original filename is not included in the header.
-            
+
             The mode argument can be any of 'r', 'rb', 'a', 'ab', 'w', or 'wb',
             depending on whether the file will be read or written.  The default
             is the mode of fileobj if discernible; otherwise, the default is 'rb'.
             Be aware that only the 'rb', 'ab', and 'wb' values should be used
             for cross-platform portability.
-            
+
             The compresslevel argument is an integer from 1 to 9 controlling the
             level of compression; 1 is fastest and produces the least compression,
             and 9 is slowest and produces the most compression.  The default is 9.
@@ -414,43 +489,60 @@ else:
             self.lock = threading.Semaphore()
             self.__size = None
 
+        def __repr__(self):
+            return "fabio." + gzip.GzipFile.__repr__(self)
 
-        if sys.version_info < (2, 7):
-            def getSize(self):
-                if self.__size is None:
-                    logger.debug("Measuring size of %s" % self.name)
-                    with open(self.filename, "rb") as f:
-                        f.seek(-4)
-                        self.__size = numpy.fromstring(f.read(4), dtype=numpy.uint32)
-                return self.__size
-            def setSize(self, value):
-                self.__size = value
-            size = property(getSize, setSize)
-            @property
-            def closed(self):
-                return self.fileobj is None
-
-            def seek(self, offset, whence=os.SEEK_SET):
-                """
-                Move to new file position.
-        
-                Argument offset is a byte count.  Optional argument whence defaults to
-                0 (offset from start of file, offset should be >= 0); other values are 1
-                (move relative to current position, positive or negative), and 2 (move
-                relative to end of file, usually negative, although many platforms allow
-                seeking beyond the end of a file).  If the file is opened in text mode,
-                only offsets returned by tell() are legal.  Use of other offsets causes
-                undefined behavior.
-                
-                This is a wrapper for seek to ensure compatibility with old python 2.5
-                """
-                if whence == os.SEEK_SET:
-                    gzip.GzipFile.seek(self, offset)
-                elif whence == os.SEEK_CUR:
-                        gzip.GzipFile.seek(self, offset + self.tell())
-                elif whence == os.SEEK_END:
-                        gzip.GzipFile.seek(self, -1)
-                        gzip.GzipFile.seek(self, offset + self.tell())
+        def measureSize(self):
+            if self.mode == gzip.WRITE:
+                return self.size
+            if self.__size is None:
+                with self.lock:
+                    if self.__size is None:
+                        pos = self.offset
+                        end_pos = len(gzip.GzipFile.read(self)) + pos
+                        self.seek(pos)
+                        logger.debug("Measuring size of %s: %s @ %s == %s" % (self.name, end_pos, pos, self.offset))
+                        self.__size = end_pos
+            return self.__size
+#        if sys.version_info < (2, 7):
+#            def setSize(self, value):
+#                self.__size = value
+#            size = property(getSize, setSize)
+#            @property
+#            def closed(self):
+#                return self.fileobj is None
+#
+#            def seek(self, offset, whence=os.SEEK_SET):
+#                """
+#                Move to new file position.
+#
+#                Argument offset is a byte count.  Optional argument whence defaults to
+#                0 (offset from start of file, offset should be >= 0); other values are 1
+#                (move relative to current position, positive or negative), and 2 (move
+#                relative to end of file, usually negative, although many platforms allow
+#                seeking beyond the end of a file).  If the file is opened in text mode,
+#                only offsets returned by tell() are legal.  Use of other offsets causes
+#                undefined behavior.
+#
+#                This is a wrapper for seek to ensure compatibility with old python 2.5
+#
+#                Warning: Seek from end is not supported (works only for single blocks !!!)
+#                This implemtents a hack
+#                """
+#                if whence == os.SEEK_SET:
+#                    gzip.GzipFile.seek(self, offset)
+#                elif whence == os.SEEK_CUR:
+#                    gzip.GzipFile.seek(self, offset + self.tell())
+#                elif whence == os.SEEK_END:
+#                    gzip.GzipFile.seek(self, offset + self.getSize())
+#
+#            def __enter__(self, *args, **kwargs):
+#                return self
+#            def __exit__(self, *args, **kwargs):
+#                """
+#                Close the file.
+#                """
+#                return gzip.GzipFile.close(self)
 
 if bz2 is None:
     BZ2File = UnknownCompressedFile
@@ -460,13 +552,13 @@ else:
         def __init__(self, name , mode='r', buffering=0, compresslevel=9):
             """
             BZ2File(name [, mode='r', buffering=0, compresslevel=9]) -> file object
-            
+
             Open a bz2 file. The mode can be 'r' or 'w', for reading (default) or
             writing. When opened for writing, the file will be created if it doesn't
             exist, and truncated otherwise. If the buffering argument is given, 0 means
             unbuffered, and larger numbers specify the buffer size. If compresslevel
             is given, must be a number between 1 and 9.
-            
+
             Add a 'U' to mode to open the file for input with universal newline
             support. Any line ending in the input file will be seen as a '\n' in
             Python. Also, a file so opened gains the attribute 'newlines'; the value
@@ -489,3 +581,47 @@ else:
         def setSize(self, value):
             self.__size = value
         size = property(getSize, setSize)
+        def __exit__(self, *args, **kwargs):
+                """
+                Close the file.
+                """
+                return bz2.BZ2File.close(self)
+        def __enter__(self, *args, **kwargs):
+            return self
+
+
+
+class DebugSemaphore(threading._Semaphore):
+    """
+    threading.Semaphore like class with helper for fighting dead-locks
+    """
+    write_lock = threading._Semaphore()
+    blocked = []
+    def __init__(self, *arg, **kwarg):
+        threading._Semaphore.__init__(self, *arg, **kwarg)
+
+
+    def acquire(self, *arg, **kwarg):
+        if self._Semaphore__value == 0:
+            with self.write_lock:
+                self.blocked.append(id(self))
+                sys.stderr.write(os.linesep.join(["Blocking sem %s" % id(self)] + \
+                                        traceback.format_stack()[:-1] + [""]))
+
+        return threading._Semaphore.acquire(self, *arg, **kwarg)
+
+    def release(self, *arg, **kwarg):
+        with self.write_lock:
+            uid = id(self)
+            if uid in self.blocked:
+                self.blocked.remove(uid)
+                sys.stderr.write("Released sem %s %s" % (uid, os.linesep))
+        threading._Semaphore.release(self, *arg, **kwarg)
+
+    def __enter__(self):
+        self.acquire()
+        return self
+
+    def __exit__(self, *arg, **kwarg):
+        self.release()
+
